@@ -56,15 +56,14 @@ class Device(Events):
     __syslog_save_path = "./syslog/"
     __syslog_save_name = ""
     __syslog_save_keyword = ""
-    __runner_log = True
     watcher_list = []
     __main_conn, __watcher_conn = Pipe()
     __width = 0
     __height = 0
+    __syslog_tid = None
 
-    def __init__(self, host="192.168.100.100", port=10008, runner_log=True, main=True):
+    def __init__(self, host="192.168.100.100", port=10008, main=True):
         super().__init__(d=self)
-        self.__runner_log = runner_log
         self.con.host = host
         self.con.port = port
         self.con.connect()
@@ -77,7 +76,6 @@ class Device(Events):
             syslog_thread.setDaemon(True)
             syslog_thread.start()
         self.refresh_layout()
-        self.log("debug", "Device, create Device object, host=%s, port=%s" % (host, str(port)))
 
     def push_watcher_data(self):
         self.__main_conn.send({'watcher_list': self.watcher_list})
@@ -86,13 +84,21 @@ class Device(Events):
         w = Watcher({'name': name}, self)
         return w
 
+    def __logger_pingpong(self):
+        self.con.get(path="SSEPingpong", args="tid=" + str(self.__syslog_tid))
+        threading.Timer(5, self.__logger_pingpong).start()
+
     def __logger(self):
         syslog_save_path = ""
         syslog_file = None
-        self.log("debug", "__logger(), logger started.")
+        first = True
         messages = self.device.con.sse("SysLogger")
         for msg in messages:
             log_str = str(msg.data)
+            if first:
+                self.__syslog_tid = log_str
+                threading.Timer(5, self.__logger_pingpong).start()
+                first = False
             if self.__syslog_output and re.search(self.__syslog_output_keyword, log_str):
                 print(log_str)
             if self.__syslog_save and re.search(self.__syslog_save_keyword, log_str):
@@ -107,10 +113,8 @@ class Device(Events):
     def set_syslog_output(self, is_enable, keyword=""):
         self.__syslog_output_keyword = keyword
         self.__syslog_output = is_enable
-        self.log("debug", "set_syslog_output(), is_enable=%s, keyword=\"%s\"" % (str(is_enable), keyword))
 
     def syslog_output(self):
-        self.log("debug", "syslog_output(), is_enable=%s" % (str(self.__syslog_output)))
         return self.__syslog_output
 
     def syslog_output_keyword(self):
@@ -193,14 +197,4 @@ class Device(Events):
 
     def find_item_by_xpath(self, sopid, xpath):
         i = Item(d=self, s=sopid, xpath=xpath)
-        self.device.log("debug", "find_item_by_xpath(), find Item successful, %s" % (str(i)))
         return i
-
-    def log(self, level, text):
-        pass
-        # if not self.__runner_log:
-        #     return
-        # local_time = time.strftime("%Y%m%d-%H:%M:%S")
-        # remote_time_raw_str = str(self.con.get(path="getSystemTime").read(), 'utf-8')[:-3]
-        # remote_time = "|R" + remote_time_raw_str[0:8] + "-" + remote_time_raw_str[8:10] + ":" + remote_time_raw_str[10:12] + ":" + remote_time_raw_str[12:14]
-        # print("[L%s%s]:%s:%s" % (local_time, remote_time, level, text))
