@@ -74,6 +74,7 @@ class Item:
         self.__scale = float(self.node.get("scale"))
         self.__rotation = int(self.node.get("rotation"))
         self.__clip = bool(int(self.node.get("clip")))
+        self.__has_contents = bool(int(self.node.get("hasContents")))
 
     def __init_attribute(self):
         self.__text = ""
@@ -96,6 +97,7 @@ class Item:
         self.__scale = 0.0
         self.__rotation = 0
         self.__clip = None
+        self.__has_contents = None
 
     def x(self, refresh=False):
         if refresh:
@@ -201,71 +203,69 @@ class Item:
         if timeout is None:
             timeout = self.device.default_timeout
         sleep(3)
-        die_time = int(time.time()) + timeout
-        while int(time.time()) < die_time:
-            self.rect = self.__exist()
-            if len(self.rect) > 0:
-                return True
-            sleep(0.5)
+        tree = xml.dom.minidom.parseString(self.device.xmlStr)
+        if tree.documentElement.getAttribute("sopId") == self.sopid:
+            die_time = int(time.time()) + timeout
+            while int(time.time()) < die_time:
+                self.rect = self.__exist()
+                if len(self.rect) > 0:
+                    return True
+                sleep(0.5)
         return False
 
     def __exist(self):
         self.__refresh_node()
-        if self.node is not None:
-            self.__refresh_attribute()
+        if self.node is not None and self.__visible and self.__opacity > 0 and self.__scale != 0 and self.__width > 0 and self.__height > 0:
+            image = []
             tree = xml.dom.minidom.parseString(self.device.xmlStr)
-            if tree.documentElement.getAttribute("sopId") == self.sopid:
-                image = []
-                for node in tree.documentElement.childNodes:
-                    if node.childNodes[0].nodeName == "QQuickMouseArea" and node.childNodes[0].childNodes.length == 0:
-                        surface = node.childNodes[1]
-                    else:
-                        surface = node.childNodes[0]
+            for node in tree.documentElement.childNodes:
+                for n in node.childNodes:
                     if len(image) == 0:
-                        image = self.__xml_tree_traversed(surface.childNodes, 1)
+                        image = self.__xml_tree_traversed(n.childNodes, 1)
                     else:
-                        image = self.__img_cover(image, self.__xml_tree_traversed(surface.childNodes, 1))
-                # cv2.imshow(str(self.device.system_time()), image)
-                # cv2.waitKey(0)
-                b, g, r = cv2.split(image)
-                if len(np.argwhere(r == 255)) > 0:
-                    return np.argwhere(r == 255)
+                        image = self.__img_cover(image, self.__xml_tree_traversed(n.childNodes, 1))
+            # cv2.imshow(str(self.device.system_time()), image)
+            # cv2.waitKey(0)
+            b, g, r = cv2.split(image)
+            if len(np.argwhere(r == 255)) > 0:
+                return np.argwhere(r == 255)
         return []
 
     def __xml_tree_traversed(self, nodes, fs):
         images = {}
         for node in nodes:
             clip = int(node.getAttribute("clip"))
-            scale = abs(float(node.getAttribute("scale"))*fs)
-            height = round(float(node.getAttribute("height"))*scale)
-            width = round(float(node.getAttribute("width"))*scale)
+            scale = abs(float(node.getAttribute("scale")) * fs)
+            height = round(float(node.getAttribute("height")) * scale)
+            width = round(float(node.getAttribute("width")) * scale)
             rotation = 0 - int(node.getAttribute("rotation"))
+            has_contents = bool(int(node.getAttribute("hasContents")))
             cx = round(float(node.getAttribute("centerXToGlobal")))
             cy = round(float(node.getAttribute("centerYToGlobal")))
             if height > 10000 or width > 10000:
                 x = float(node.getAttribute("x"))
                 y = float(node.getAttribute("y"))
-                cx, cy, width, height = self.__clip_item(float(cx), float(cy), float(width), float(height), x, y, 0-rotation)
+                cx, cy, width, height = self.__clip_item(float(cx), float(cy), float(width), float(height), x, y, 0 - rotation)
             if float(node.getAttribute("opacity")) != 0 \
                     and int(node.getAttribute("visible")) == 1 \
-                    and node.nodeName != "CScrollDecorator" \
-                    and node.nodeName != "QQuickShaderEffectSource" \
-                    and node.nodeName != "PerformanceOverlay" \
-                    and node.nodeName != "QQuickDropArea" \
-                    and height > 0 and width > 0 \
-                    and not (node.nodeName == "QQuickMouseArea" and node.childNodes.length == 0) \
-                    and not (
-                        node.getAttribute("objectName") == "multitouchGrid" and int(node.getAttribute("enabled")) == 0):
+                    and node.nodeName != "QQuickShaderEffectSource":
                 index_z = int(node.getAttribute("z"))
-                image = np.zeros((height, width), dtype=np.uint8)
+                if height > 0 and width > 0:
+                    image = np.zeros((height, width), dtype=np.uint8)
+                else:
+                    image = np.zeros((self.__display_height, self.__display_width), dtype=np.uint8)
                 image = cv2.cvtColor(image, cv2.COLOR_GRAY2BGR)
-                if node.nodeName != "QQuickLoader" or (node.nodeName == "QQuickLoader" and node.getAttribute("z") == "10000" and node.getAttribute("tempID") == "temp6"):
+                if has_contents or (node.nodeName == "QQuickLoader" and node.getAttribute("z") == "10000" and
+                                    node.getAttribute("tempID") == "temp7"):
                     if node.getAttribute("tempID") == self.__tempId:
                         cv2.rectangle(image, (0, 0), (width, height), (0, 0, 255), -1)
                     else:
-                        cv2.rectangle(image, (0, 0), (width, height), (255, 0, 0), -1)
-                        # cv2.rectangle(image, (0, 0), (width, height), (random.randint(0, 255), random.randint(0, 255), random.randint(0, 255)), -1)
-                hypotenuse = int(math.sqrt(height**2+width**2))
+                        # cv2.rectangle(image, (0, 0), (width, height), (255, 0, 0), -1)
+                        cv2.rectangle(image, (0, 0), (width, height),
+                                     (random.randint(0, 255), 0, random.randint(0, 255)), -1)
+                else:
+                    cv2.rectangle(image, (0, 0), (width, height), (0, 255, 0), -1)
+                hypotenuse = int(math.sqrt(height ** 2 + width ** 2))
                 M1 = np.float32([[1, 0, (hypotenuse - width) / 2], [0, 1, (hypotenuse - height) / 2]])
                 image = cv2.warpAffine(image, M1, (hypotenuse, hypotenuse))
                 M2 = cv2.getRotationMatrix2D((hypotenuse / 2, hypotenuse / 2), rotation, 1)
@@ -511,11 +511,13 @@ class Item:
             lyb = (l2y - l1y) / (l2x - l1x)
         if sy <= l1y + (0 - l1x) * lyb <= ey and 0 <= l1y + (0 - l1x) * lyb <= self.__display_height:
             return True, 0, l1y + (0 - l1x) * lyb
-        if sy <= l1y + (self.__display_width - l1x) * lyb <= ey and 0 <= l1y + (self.__display_width - l1x) * lyb <= self.__display_height:
+        if sy <= l1y + (self.__display_width - l1x) * lyb <= ey and 0 <= l1y + (
+                self.__display_width - l1x) * lyb <= self.__display_height:
             return True, self.__display_width, l1y + (self.__display_width - l1x) * lyb
         if sx <= l1x + (0 - l1y) * lxb <= ex and 0 <= l1x + (0 - l1y) * lxb <= self.__display_width:
             return True, l1x + (0 - l1y) * lxb, 0
-        if sx <= l1x + (self.__display_height - l1y) * lxb <= ex and 0 <= l1x + (self.__display_height - l1y) * lxb <= self.__display_width:
+        if sx <= l1x + (self.__display_height - l1y) * lxb <= ex and 0 <= l1x + (
+                self.__display_height - l1y) * lxb <= self.__display_width:
             return True, l1x + (self.__display_height - l1y) * lxb, self.__display_height
         return False, 0, 0
 
@@ -538,16 +540,21 @@ class Item:
     @staticmethod
     def __img_cover(src1, src2, clip=0):
         rows, cols, channels = src2.shape
-        roi = src1[0:rows, 0:cols]
         roi2 = src2[0:rows, 0:cols]
         if clip == 1:
             src1_2gray = cv2.cvtColor(src1, cv2.COLOR_BGR2GRAY)
             ret1, mask1 = cv2.threshold(src1_2gray, 1, 255, cv2.THRESH_BINARY_INV)
             image_bg1 = cv2.bitwise_and(roi2, roi2, mask=mask1)
             src2 = cv2.subtract(src2, image_bg1)
+        B1, G1, R1 = cv2.split(src1)
+        zeros = np.zeros(src1.shape[:2], dtype="uint8")
+        src1 = cv2.merge([B1, zeros, R1])
+        roi1 = src1[0:rows, 0:cols]
+        B2, G2, R2 = cv2.split(src2)
+        src2 = cv2.merge([B2, zeros, R2])
         src2_2gray = cv2.cvtColor(src2, cv2.COLOR_BGR2GRAY)
-        ret, mask = cv2.threshold(src2_2gray, 1, 255, cv2.THRESH_BINARY)
-        image_bg = cv2.bitwise_and(roi, roi, mask=mask)
+        ret2, mask2 = cv2.threshold(src2_2gray, 1, 255, cv2.THRESH_BINARY)
+        image_bg = cv2.bitwise_and(roi1, roi1, mask=mask2)
         image = cv2.subtract(src1, image_bg)
         image = cv2.add(image, src2)
         return image
